#ifndef HASHMANAGER_H
#define HASHMANAGER_H


/**
 *    This class stores 6 hash maps used for MAC filters, of for each possible
 *    MAC filter length.
 *
 *    Full MAC addresses will get stored in the [6] table, whereas smaller
 *    MAC filters (00:11:22, 00:11:33:44:55) will be stored in tables of
 *    their own size.  The lookup() function searches all tables with an
 *    emphasis on speed.
 *
 *    @author Mark Manes <mmanes@stillsecure.com>
 */


#include <stdio.h>
#include "HwAddrHash.h"

class HashManager
{
public:
    HashManager(int minBuckets);
    virtual ~HashManager();
    typedef vector<HwAddr> AddrList;
    
    // Put key into hash of it's corresponding size
    void put(const HwAddr &hw) throw(std::invalid_argument);
    
    // Remove HwAddr from hash; returns bool if HwAddr was present.
    bool remove(const HwAddr &hw) throw(std::invalid_argument);
    
    // Returns true if hw exists and returns value in val
    //bool get(const HwAddr &hw) const throw(std::invalid_argument);
    
    // Returns true if hw exists in
    bool exists(const HwAddr &hw) throw(std::invalid_argument);
    
    // Returns true if hw exists in any of the tables, overwrites hw with key in map
    bool lookup(HwAddr &hw) const throw(std::invalid_argument);
    
    // Returns all values from all hashes
    bool getAll(HwAddrHash::AddrList &alist, int size = -1) const;
    
    // Clear ALL values in ALL hashes
    void clear();
    
    // Generate a string containing statistics about the hash
    string generateStats(int size = -1) const;
    
#ifdef UNITTEST
    static void test() throw(std::logic_error);
#endif
    
private:
    HwAddrHash* hashes[6];
    HwAddrHash* getHash(const HwAddr &hw) throw(std::invalid_argument);
    HwAddrHash* getHash(int size) throw(std::invalid_argument);
};

#endif
